# Copyright 2020 All right reserved
# Author: Chester Chee <chester.chee@gmail.com>
#
# Field byte size constants
SESSION_FIELD_LEN       = 10
SEQ_NUM_FIELD_LEN       = 8
MESSAGE_COUNT_FIELD_LEN = 2
MESSAGE_SIZE_FIELD_LEN  = 2

# Position constants
SESSION_OFFSET = 0
SEQ_NUM_OFFSET = SESSION_OFFSET + SESSION_FIELD_LEN
MSG_CNT_OFFSET = SEQ_NUM_OFFSET + SEQ_NUM_FIELD_LEN
PAYLOAD_OFFSET = MSG_CNT_OFFSET + MESSAGE_COUNT_FIELD_LEN

# Packet byte size constants
MTU_SIZE     = 1500
UDP_HDR_SIZE = 8
HEADER_SIZE  = PAYLOAD_OFFSET
PAYLOAD_SIZE = MTU_SIZE - UDP_HDR_SIZE - HEADER_SIZE
MOLDPKT_SIZE = HEADER_SIZE + PAYLOAD_SIZE

# Packet Type constants
HEART_BEAT     = 0
END_OF_SESSION = 65535 # hex: 0xFFFF



from setuptools import setup, find_packages
from os import path

here = path.abspath(path.dirname(__file__))

with open(path.join(here, "README.md"), "r") as fh:
    long_description = fh.read()


setup(
    name='moldudp',
    version='0.0.16',
    description='A simple library to decode/encode MoldUDP64 bytearray/packet',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/cchee/moldudp',
    author='Chester Chee',
    author_email='chester.chee@gmail.com',
    packages=find_packages(),
    python_requires='>=3.0',
    project_urls={
        'Bug Reports': 'https://github.com/cchee/moldudp/issues',
        'Source': 'https://github.com/cchee/moldudp/',
    },
    classifiers=[
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    keywords='moldudp nasdaq development',
    license=open('LICENSE').read(),
    platform='any',
)



# Copyright 2020 All right reserved
# Author: Chester Chee <chester.chee@gmail.com>
#
# MoldUDP bytearray/packet decoder according to the specification from Nasdaq
# http://www.nasdaqtrader.com/content/technicalsupport/specifications/dataproducts/moldudp64.pdf
#
from struct import unpack_from
from struct import calcsize

from moldudp.codec.const import END_OF_SESSION
from moldudp.codec.const import HEART_BEAT
from moldudp.codec.const import PAYLOAD_OFFSET
from moldudp.codec.const import SESSION_OFFSET
from moldudp.codec.const import MESSAGE_SIZE_FIELD_LEN
from moldudp.codec.msgsub import MsgSubscriber


class MoldUDPDecoder:

    # subscriber is the application message processor
    def __init__(self, subscriber: MsgSubscriber, debug=False):
        self._offset = SESSION_OFFSET
        self._subscriber = subscriber
        self._debug = debug

    def _process_msghdr(self, session, seq, msgcount):
        self._session = session
        self._seq = seq
        self._msgcount = msgcount
        if (self._debug == True):
            print("RECEIVED HDR SESSION: {}".format(self._session))
            print("RECEIVED HDR SEQ:     {}".format(self._seq))
            print("RECEIVED MSG COUNT:   {}".format(self._msgcount))

    def _process_msgblks(self):
        i = self._msgcount
        while (i > 0):
            # message block consists of 2-bytes message length and follows by message data with previously specified length
            blk_sz = unpack_from('>H', self._buffer, self._offset)[0]
            self._offset += MESSAGE_SIZE_FIELD_LEN
            fmt = '>{}s'.format(blk_sz)
            msg = unpack_from(fmt, self._buffer, self._offset)[0]
            if (self._subscriber):
                self._subscriber.on_msgblk(msg)
            self._offset += calcsize(fmt)
            i -= 1

    def buffer(self, buffer):
        self._buffer = bytearray(buffer)
        (session, seq, msgcount) = unpack_from('>10sQh', self._buffer)
        self._process_msghdr(session, seq, msgcount)
        self._offset = PAYLOAD_OFFSET

    def decode(self):
        if (self._subscriber == None):
            return

        if (self.is_hb() == True):
            self._subscriber.on_hb()
        elif (self.is_eos() == True):
            self._subscriber.on_eos()
        else:
            self._process_msgblks()

    def session(self):
        return self._session

    def seq(self):
        return self._seq

    def count(self):
        return self._msgcount

    def is_hb(self):
        return self._msgcount == HEART_BEAT

    def is_eos(self):
        return self._msgcount == END_OF_SESSION
